@@ -171,10 +171,6 @@ present for backward compatibility."
     tramp-efs-method
     '((tramp-parse-netrc "~/.netrc"))))
 
-;; If there is URL syntax, `substitute-in-file-name' needs special
-;; handling.
-(put 'substitute-in-file-name 'efs 'tramp-handle-substitute-in-file-name)
-
 ;;;###tramp-autoload
 (defun tramp-efs-file-name-handler (operation &rest args)
   "Invoke the EFS handler for OPERATION.
