@@ -1,6 +1,6 @@
 ;;; tramp-gw.el --- Tramp utility functions for HTTP tunnels and SOCKS gateways
 
-;; Copyright (C) 2007-2013 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2014 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -96,7 +96,7 @@
   (unless (memq (process-status proc) '(run open))
     (tramp-message
      tramp-gw-vector 4 "Deleting auxiliary process `%s'" tramp-gw-gw-proc)
-    (let* (tramp-verbose
+    (let* ((tramp-verbose 0)
 	   (p (tramp-get-connection-property proc "process" nil)))
       (when (processp p) (delete-process p)))))
 
@@ -111,7 +111,7 @@
     (tramp-compat-set-process-query-on-exit-flag proc nil)
     ;; We don't want debug messages, because the corresponding debug
     ;; buffer might be undecided.
-    (let (tramp-verbose)
+    (let ((tramp-verbose 0))
       (tramp-set-connection-property tramp-gw-gw-proc "process" proc)
       (tramp-set-connection-property proc "process" tramp-gw-gw-proc))
     ;; Set the process-filter functions for both processes.
@@ -125,7 +125,7 @@
 	  (tramp-gw-process-filter tramp-gw-gw-proc s))))))
 
 (defun tramp-gw-process-filter (proc string)
-  (let (tramp-verbose)
+  (let ((tramp-verbose 0))
     (process-send-string
      (tramp-get-connection-property proc "process" nil) string)))
 
@@ -195,11 +195,12 @@ instead of the host name declared in TARGET-VEC."
     (setq tramp-gw-gw-proc
 	  (funcall
 	   socks-function
-	   (tramp-get-connection-name gw-vec)
-	   (tramp-get-connection-buffer gw-vec)
+	   (let ((tramp-verbose 0)) (tramp-get-connection-name gw-vec))
+	   (let ((tramp-verbose 0)) (tramp-get-connection-buffer gw-vec))
 	   (tramp-file-name-real-host target-vec)
 	   (tramp-file-name-port target-vec)))
     (set-process-sentinel tramp-gw-gw-proc 'tramp-gw-gw-proc-sentinel)
+    (set-process-coding-system tramp-gw-gw-proc 'binary 'binary)
     (tramp-compat-set-process-query-on-exit-flag tramp-gw-gw-proc nil)
     (tramp-message
      vec 4 "Opened %s process `%s'"
@@ -245,7 +246,7 @@ authentication is requested from proxy server, provide it."
 	;; proxies have a timeout of 60".  We wait 65" in order to
 	;; receive an answer this case.
 	(ignore-errors
-	  (let (tramp-verbose)
+	  (let ((tramp-verbose 0))
 	    (tramp-wait-for-regexp proc 65 "\r?\n\r?\n")))
 	;; Check return code.
 	(goto-char (point-min))
@@ -260,6 +261,10 @@ authentication is requested from proxy server, provide it."
 	  (200 (setq found t))
 	  ;; We need basic authentication.
 	  (401 (setq authentication (tramp-gw-basic-authentication nil first)))
+	  ;; Access forbidden.
+	  (403 (tramp-error-with-buffer
+		(current-buffer) tramp-gw-vector 'file-error
+		"Connection to %s:%d forbidden." host service))
 	  ;; Target host not found.
 	  (404 (tramp-error-with-buffer
 		(current-buffer) tramp-gw-vector 'file-error
