@@ -513,7 +513,7 @@ pass to the OPERATION."
 		;; Reset the transfer process properties.
 		(tramp-set-connection-property v "process-name" nil)
 		(tramp-set-connection-property v "process-buffer" nil)
-		(when t1 (delete-directory tmpdir 'recurse))))
+		(when t1 (tramp-compat-delete-directory tmpdir 'recurse))))
 
 	    ;; Handle KEEP-DATE argument.
 	    (when keep-date
@@ -976,7 +976,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		 (lambda (x y)
 		   (if (string-match "t" switches)
 		       ;; Sort by date.
-		       (tramp-time-less-p (nth 3 y) (nth 3 x))
+		       (time-less-p (nth 3 y) (nth 3 x))
 		     ;; Sort by name.
 		     (string-lessp (nth 0 x) (nth 0 y))))))
 
@@ -1010,8 +1010,7 @@ PRESERVE-UID-GID and PRESERVE-EXTENDED-ATTRIBUTES are completely ignored."
 		     (or (nth 3 attr) "nogroup") ; gid
 		     (or (nth 7 attr) (nth 2 x)) ; size
 		     (format-time-string
-		      (if (tramp-time-less-p
-			   (tramp-time-subtract (current-time) (nth 3 x))
+		      (if (time-less-p (time-subtract (current-time) (nth 3 x))
 			   tramp-half-a-year)
 			  "%b %e %R"
 			"%b %e  %Y")
@@ -1225,8 +1224,8 @@ target of the symlink differ."
 	(error
 	 (setq ret 1)))
 
-      ;; We should show the output anyway.
-      (when (and outbuf display) (display-buffer outbuf))
+      ;; We should redisplay the output.
+      (when (and display outbuf (get-buffer-window outbuf t)) (redisplay))
 
       ;; Cleanup.  We remove all file cache values for the connection,
       ;; because the remote process could have changed them.
