@@ -116,18 +116,15 @@ shall not contain a timeout."
   `(let ((tramp-verbose ,verbose)
 	 (tramp-message-show-message t)
 	 (tramp-debug-on-error t))
-     (condition-case err
+     (unwind-protect
 	 (progn ,@body)
-       (ert-test-skipped
-	(signal (car err) (cdr err)))
-       ((error quit)
-	(with-parsed-tramp-file-name tramp-test-temporary-file-directory nil
-	  (with-current-buffer (tramp-get-connection-buffer v)
-	    (message "%s" (buffer-string)))
-	  (with-current-buffer (tramp-get-debug-buffer v)
-	    (message "%s" (buffer-string))))
-	(message "%s" err)
-	(signal (car err) (cdr err))))))
+       (when (> tramp-verbose 3)
+	 (with-parsed-tramp-file-name tramp-test-temporary-file-directory nil
+	   (with-current-buffer (tramp-get-connection-buffer v)
+	     (message "%s" (buffer-string)))
+	   (with-current-buffer
+	       (tramp-get-debug-buffer v)
+	     (message "%s" (buffer-string))))))))
 
 (ert-deftest tramp-test00-availability ()
   "Test availability of Tramp functions."
@@ -956,7 +953,12 @@ This tests also `file-directory-p' and `file-accessible-directory-p'."
 	    (goto-char (point-min))
 	    (should
 	     (looking-at-p
-	      "\\(total.+[[:digit:]]+\n\\)?.+ \\.\n.+ \\.\\.\n.+ foo$"))))
+	      (concat
+	       ;; There might be a summary line.
+	       "\\(total.+[[:digit:]]+\n\\)?"
+	       ;; We don't know in which order "." and ".." appear.
+	       "\\(.+ \\.?\\.\n\\)\\{2\\}"
+	       ".+ foo$")))))
       (ignore-errors (delete-directory tmp-name1 'recursive)))))
 
 (ert-deftest tramp-test18-file-attributes ()
@@ -1246,9 +1248,10 @@ This tests also `make-symbolic-link', `file-truename' and `add-name-to-file'."
      (tramp-find-foreign-file-name-handler tramp-test-temporary-file-directory)
      '(tramp-gvfs-file-name-handler tramp-smb-file-name-handler))))
 
-  (let ((tmp-name (tramp--test-make-temp-name))
-	(default-directory tramp-test-temporary-file-directory)
-	kill-buffer-query-functions)
+  (let* ((tmp-name (tramp--test-make-temp-name))
+	 (fnnd (file-name-nondirectory tmp-name))
+	 (default-directory tramp-test-temporary-file-directory)
+	 kill-buffer-query-functions)
     (unwind-protect
 	(progn
 	  ;; We cannot use "/bin/true" and "/bin/false"; those paths
@@ -1259,17 +1262,25 @@ This tests also `make-symbolic-link', `file-truename' and `add-name-to-file'."
 	  (with-temp-buffer
 	    (write-region "foo" nil tmp-name)
 	    (should (file-exists-p tmp-name))
-	    (should
-	     (zerop
-	      (process-file "ls" nil t nil (file-name-nondirectory tmp-name))))
+	    (should (zerop (process-file "ls" nil t nil fnnd)))
+	    ;; `ls' could produce colorized output.
+	    (goto-char (point-min))
+	    (while (re-search-forward tramp-color-escape-sequence-regexp nil t)
+	      (replace-match "" nil nil))
+	    (should (string-equal (format "%s\n" fnnd) (buffer-string)))
+	    (should-not (get-buffer-window (current-buffer) t))
+
+	    ;; Second run. The output must be appended.
+	    (should (zerop (process-file "ls" nil t t fnnd)))
 	    ;; `ls' could produce colorized output.
 	    (goto-char (point-min))
 	    (while (re-search-forward tramp-color-escape-sequence-regexp nil t)
 	      (replace-match "" nil nil))
 	    (should
-	     (string-equal
-	      (format "%s\n" (file-name-nondirectory tmp-name))
-	      (buffer-string)))))
+	     (string-equal (format "%s\n%s\n" fnnd fnnd) (buffer-string)))
+	    ;; A non-nil DISPLAY must not raise the buffer.
+	    (should-not (get-buffer-window (current-buffer) t))))
+
       (ignore-errors (delete-file tmp-name)))))
 
 (ert-deftest tramp-test27-start-file-process ()
@@ -1463,9 +1474,15 @@ This tests also `make-symbolic-link', `file-truename' and `add-name-to-file'."
 	    (vc-create-repo (car vc-handled-backends))
 	    ;; The structure of VC-FILESET is not documented.  Let's
 	    ;; hope it won't change.
-	    (vc-register
-	     nil (list (car vc-handled-backends)
-		       (list (file-name-nondirectory tmp-name2)))))
+	    (condition-case nil
+		(vc-register
+		 (list (car vc-handled-backends)
+		       (list (file-name-nondirectory tmp-name2))))
+	      ;; `vc-register' has changed its arguments in Emacs 25.1.
+	      (error
+	       (vc-register
+		nil (list (car vc-handled-backends)
+			  (list (file-name-nondirectory tmp-name2)))))))
 	  (should (vc-registered tmp-name2)))
 
 	(ignore-errors (delete-directory tmp-name1 'recursive)))))
@@ -1536,6 +1553,8 @@ This tests also `make-symbolic-link', `file-truename' and `add-name-to-file'."
      '(tramp-adb-file-name-handler
        tramp-gvfs-file-name-handler
        tramp-smb-file-name-handler))))
+  ;; Bug#19463.
+  (skip-unless (not (eq system-type 'windows-nt)))
 
   ;; Newlines, slashes and backslashes in file names are not supported.
   ;; So we don't test.
@@ -1731,10 +1750,9 @@ Since it unloads Tramp, it shall be the last test to run."
 ;;   doesn't work well when an interactive password must be provided.
 ;; * Fix `tramp-test27-start-file-process' for `nc' and on MS
 ;;   Windows (`process-send-eof'?).
-;; * Fix `tramp-test28-shell-command' on MS Windows (nasty plink message).
 ;; * Fix `tramp-test30-special-characters' for `adb', `nc' and `smb'.
-;; * Fix `tramp-test31-utf8' for MS Windows and `nc'/`telnet' (when
-;;   target is a dumb busybox).  Seems to be in `directory-files'.
+;; * Fix `tramp-test31-utf8' for `nc'/`telnet' (when target is a dumb
+;;   busybox).  Seems to be in `directory-files'.
 ;; * Fix Bug#16928.  Set expected error of `tramp-test32-asynchronous-requests'.
 ;; * Fix `tramp-test34-unload' (Not all symbols are unbound).  Set
 ;;   expected error.
